﻿using System;
using System.Collections.Generic;
using System.Web;
using Libs;
using WRECWrapper.Responses;

namespace WRECWrapper
{
    public class ActivateService
    {
        public enum ContractState
        {
            PENDING_ACTIVATION,
            PENDING_DEACTIVATION
        }
        private readonly string m_token;
        private readonly string m_connectorURL;

        public ActivateService(string token, string connectorURL)
        {
            m_token = token;
            m_connectorURL = connectorURL;
        }

        public IList<MediaContractSnapshot> GetMediaContractSnapshots(ContractState contractState)
        {
            return getMediaContractSnapshots(contractStateToString(contractState));
        }
        public IList<MediaContractSnapshot> GetMediaContractSnapshots()
        {
            return getMediaContractSnapshots(string.Empty);
        }
        
        public void AcceptActivation(int mediaContractID)
        {
            var url = string.Format("{0}ActivateService.svc/1/0/{1}/mediacontract/?id={2}&action={3}",
                m_connectorURL, m_token, mediaContractID, "acceptactivation");
            
            var result = HttpLib.PostMessage<DefaultResult>(url);
            if ((result == null) || (!result.IsSuccess))
                throw new Exception((result == null) ? "Invalid response." : result.ErrorMessage ?? "Unknown error message.");
        }
        public void RejectActivation(int mediaContractID, string reason)
        {
            var url = string.Format("{0}ActivateService.svc/1/0/{1}/mediacontract/?id={2}&action={3}&reason={4}",
                m_connectorURL, m_token, mediaContractID, "rejectactivation", HttpUtility.UrlEncode(reason));
            
            var result = HttpLib.PostMessage<DefaultResult>(url);
            if (!result.IsSuccess)
                throw new Exception(result.ErrorMessage ?? "Unknown error message.");
        }
        public void SuspendContract(int mediaContractID, string reason)
        {
            var url = string.Format("{0}ActivateService.svc/1/0/{1}/mediacontract/?id={2}&action={3}&reason={4}",
                m_connectorURL, m_token, mediaContractID, "suspend", HttpUtility.UrlEncode(reason));
            
            var result = HttpLib.PostMessage<DefaultResult>(url);
            if (!result.IsSuccess)
                throw new Exception(result.ErrorMessage ?? "Unknown error message.");
        }
        public void ReactivateContract(int mediaContractID)
        {
            var url = string.Format("{0}ActivateService.svc/1/0/{1}/mediacontract/?id={2}&action={3}",
                m_connectorURL, m_token, mediaContractID, "reactivate");
            
            var result = HttpLib.PostMessage<DefaultResult>(url);
            if (!result.IsSuccess)
                throw new Exception(result.ErrorMessage ?? "Unknown error message.");
        }
        public void AcceptDeactivation(int mediaContractID)
        {
            var url = string.Format("{0}ActivateService.svc/1/0/{1}/mediacontract/?id={2}&action={3}",
                m_connectorURL, m_token, mediaContractID, "acceptdeactivation");
            
            var result = HttpLib.PostMessage<DefaultResult>(url);
            if (!result.IsSuccess)
                throw new Exception(result.ErrorMessage ?? "Unknown error message.");
        }
        
        private IList<MediaContractSnapshot> getMediaContractSnapshots(string constractState)
        {
            var url = string.Format(@"{0}ActivateService.svc/1/0/{1}/mediacontract/?state={2}",
                m_connectorURL, m_token, constractState);
            var result = XmlLib.GetObject<MediaContractSnapshotResult>(url);
            if (!result.IsSuccess)
                throw new Exception(result.ErrorMessage);
            return result.ArrayOfMediaContractSnapshot;
        }
       
        private static string contractStateToString(ContractState contractState)
        {
            switch (contractState)
            {
                case ContractState.PENDING_ACTIVATION:
                    return "pendingactivation";
                case ContractState.PENDING_DEACTIVATION:
                    return "pendingdeactivation";
            }
            return string.Empty;
        }
    }
}
